@@ -1,7 +1,8 @@
+# -*- coding: utf-8 -*-
 #
 # path.py
 #
-# Copyright (C) 2017-2020 frnmst (Franco Masotti) <franco.masotti@live.com>
+# Copyright (C) 2017-2020 Franco Masotti (see /README.md)
 #
 # This file is part of fpyutils.
 #
@@ -79,18 +80,18 @@ def gen_pseudorandom_path(path_suffix: str = str(),
     """
     # 1. the current date.
     # call the fpyutils.datetime module.
-    date_component = datetime.date.strftime(datetime.datetime.now(),
-                                            date_component_format)
+    date_component: str = datetime.date.strftime(datetime.datetime.now(),
+                                                 date_component_format)
 
     # 2. a pseudorandom component.
-    pseudorandom_component = secrets.token_urlsafe(
+    pseudorandom_component: str = secrets.token_urlsafe(
         nbytes=pseudorandom_component_bytes)
 
     # 3. a hash of path_suffix. This will be equal to
     #    'cec7ea' using blake2b and a digest size of 3.
-    m = hashlib.blake2b(digest_size=hash_component_digest_size)
-    m.update(path_suffix.encode(character_encoding))
-    hashed_component = m.hexdigest()
+    hashed_component: str = (hashlib.blake2b(
+        path_suffix.encode(character_encoding),
+        digest_size=hash_component_digest_size).hexdigest())
 
     # 4. the path suffix, if present.
     if path_suffix != str():
